/**
 * @file
 * VuoList_VuoMesh C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMesh_H
#define VuoList_VuoMesh_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMesh VuoList_VuoMesh
 * A list of @ref VuoMesh elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMesh_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMesh elements.
 */
typedef const struct VuoList_VuoMesh_struct { void *l; } * VuoList_VuoMesh;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMesh elements.
 */
VuoList_VuoMesh VuoListCreate_VuoMesh(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoMesh to quickly initialize a large list.
 */
VuoList_VuoMesh VuoListCreateWithCount_VuoMesh(const unsigned long count, const VuoMesh value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMesh VuoListCopy_VuoMesh(const VuoList_VuoMesh list);

/**
 * Returns the @ref VuoMesh at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMesh VuoListGetValue_VuoMesh(const VuoList_VuoMesh list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoMesh *VuoListGetData_VuoMesh(const VuoList_VuoMesh list);

/**
 * Changes the @ref VuoMesh at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMesh(const VuoList_VuoMesh list, const VuoMesh value, const unsigned long index);

/**
 * Inserts the `VuoMesh` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMesh(const VuoList_VuoMesh list, const VuoMesh value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMesh(VuoList_VuoMesh list, const VuoMesh value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMesh(VuoList_VuoMesh list, const VuoMesh value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMesh(VuoList_VuoMesh list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoMesh_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoMesh(VuoList_VuoMesh list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoMesh_areEqual(const VuoList_VuoMesh a, const VuoList_VuoMesh b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoMesh_isLessThan(const VuoList_VuoMesh a, const VuoList_VuoMesh b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMesh(VuoList_VuoMesh list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMesh(VuoList_VuoMesh list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMesh(VuoList_VuoMesh list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMesh(VuoList_VuoMesh list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMesh(VuoList_VuoMesh list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMesh(VuoList_VuoMesh list);

/**
 * Removes the `VuoMesh` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMesh(VuoList_VuoMesh list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMesh(const VuoList_VuoMesh list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMesh VuoList_VuoMesh_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMesh_getJson(const VuoList_VuoMesh value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMesh_getSummary(const VuoList_VuoMesh value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMesh VuoList_VuoMesh_makeFromString(const char *str);
char * VuoList_VuoMesh_getString(const VuoList_VuoMesh value);
/// @}

/**
 * @}
 */

#endif
